@@ -1,5 +1,6 @@
 #!/bin/python
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     from picomc import main
+
     main()
