@@ -7,15 +7,16 @@ import shutil
 import urllib.parse
 import urllib.request
 from functools import reduce
+from pathlib import PurePath
 
 import requests
+
 from picomc.downloader import DownloadQueue
-from picomc.env import Env, get_filepath
-from picomc.javainfo import get_java_info
+from picomc.java import get_java_info
 from picomc.library import Library
 from picomc.logging import logger
 from picomc.rules import match_ruleset
-from picomc.utils import die, file_sha1, recur_files
+from picomc.utils import Directory, die, file_sha1, recur_files
 
 
 class VersionType(enum.Flag):
@@ -56,6 +57,25 @@ def argumentadd(d1, d2):
     return d
 
 
+_sentinel = object()
+
+LEGACY_ASSETS = {
+    "id": "legacy",
+    "sha1": "770572e819335b6c0a053f8378ad88eda189fc14",
+    "size": 109634,
+    "totalSize": 153475165,
+    "url": (
+        "https://launchermeta.mojang.com/v1/packages/"
+        "770572e819335b6c0a053f8378ad88eda189fc14/legacy.json"
+    ),
+}
+
+LEGACY_JAVA_VERSION = {
+    "component": "jre-legacy",
+    "majorVersion": 8,
+}
+
+
 class VersionSpec:
     def __init__(self, vobj, version_manager):
         self.vobj = vobj
@@ -71,11 +91,11 @@ class VersionSpec:
             chain.append(cv)
         return chain
 
-    def attr_override(self, attr, default=None):
+    def attr_override(self, attr, default=_sentinel):
         for v in self.chain:
             if attr in v.raw_vspec:
                 return v.raw_vspec[attr]
-        if default is None:
+        if default is _sentinel:
             raise AttributeError(attr)
         return default
 
@@ -94,9 +114,17 @@ class VersionSpec:
             self.arguments = self.attr_reduce("arguments", argumentadd)
         except AttributeError:
             pass
+
+        # TODO maybe we could detect extremely old versions of Minecraft and
+        # put an appropriate version of java here in that case
+        self.javaVersion = self.attr_override(
+            "javaVersion", default=LEGACY_JAVA_VERSION
+        )
         self.mainClass = self.attr_override("mainClass")
-        self.assetIndex = self.attr_override("assetIndex")
-        self.assets = self.attr_override("assets")
+        self.assetIndex = self.attr_override("assetIndex", default=None)
+        self.assets = self.attr_override("assets", default="legacy")
+        if self.assetIndex is None and self.assets == "legacy":
+            self.assetIndex = LEGACY_ASSETS
         self.libraries = self.attr_reduce("libraries", lambda x, y: y + x)
         self.jar = self.attr_override("jar", default=self.vobj.version_name)
         self.downloads = self.attr_override("downloads", default={})
@@ -105,39 +133,42 @@ class VersionSpec:
 class Version:
     ASSETS_URL = "http://resources.download.minecraft.net/"
 
-    def __init__(self, version_name, version_manager, version_manifest):
+    def __init__(self, version_name, launcher, version_manifest):
         self.version_name = version_name
-        self.version_manager = version_manager
+        self.launcher = launcher
+        self.vm = launcher.version_manager
         self.version_manifest = version_manifest
         self._libraries = dict()
 
+        self.versions_root = self.vm.versions_root
+        self.assets_root = self.launcher.get_path(Directory.ASSETS)
+
         self.raw_vspec = self.get_raw_vspec()
-        self.vspec = VersionSpec(self, self.version_manager)
+        self.vspec = VersionSpec(self, self.vm)
 
-        self.raw_asset_index = self.get_raw_asset_index(self.vspec)
+        if self.vspec.assetIndex is not None:
+            self.raw_asset_index = self.get_raw_asset_index(self.vspec.assetIndex)
 
-        jarname = self.vspec.jar
-        self.jarfile = get_filepath("versions", jarname, "{}.jar".format(jarname))
+        self.jarname = self.vspec.jar
+        self.jarfile = self.versions_root / self.jarname / "{}.jar".format(self.jarname)
+
+        self.java_version = self.vspec.javaVersion
 
     def get_raw_vspec(self):
-        vspec_path = get_filepath(
-            "versions", self.version_name, "{}.json".format(self.version_name)
+        vspec_path = (
+            self.versions_root / self.version_name / "{}.json".format(self.version_name)
         )
         if not self.version_manifest:
-            if os.path.exists(vspec_path):
+            if vspec_path.exists():
                 logger.debug("Found custom vspec ({})".format(self.version_name))
                 with open(vspec_path) as fp:
                     return json.load(fp)
             else:
                 die("Specified version ({}) not available".format(self.version_name))
         url = self.version_manifest["url"]
-        # Pull the hash out of the url. This is prone to breakage, maybe
-        # just try to download the vspec and don't care about whether it
-        # is up to date or not.
-        url_split = urllib.parse.urlsplit(url)
-        sha1 = posixpath.basename(posixpath.dirname(url_split.path))
+        sha1 = self.version_manifest["sha1"]
 
-        if os.path.exists(vspec_path) and file_sha1(vspec_path) == sha1:
+        if vspec_path.exists() and file_sha1(vspec_path) == sha1:
             logger.debug(
                 "Using cached vspec files, hash matches manifest ({})".format(
                     self.version_name
@@ -149,8 +180,7 @@ class Version:
         try:
             logger.debug("Downloading vspec file")
             raw = requests.get(url).content
-            dirpath = os.path.dirname(vspec_path)
-            os.makedirs(dirpath, exist_ok=True)
+            vspec_path.parent.mkdir(parents=True, exist_ok=True)
             with open(vspec_path, "wb") as fp:
                 fp.write(raw)
             j = json.loads(raw)
@@ -158,12 +188,12 @@ class Version:
         except requests.ConnectionError:
             die("Failed to retrieve version json file. Check your internet connection.")
 
-    def get_raw_asset_index(self, vspec):
-        iid = vspec.assetIndex["id"]
-        url = vspec.assetIndex["url"]
-        sha1 = vspec.assetIndex["sha1"]
-        fpath = get_filepath("assets", "indexes", "{}.json".format(iid))
-        if os.path.exists(fpath) and file_sha1(fpath) == sha1:
+    def get_raw_asset_index(self, asset_index_spec):
+        iid = asset_index_spec["id"]
+        url = asset_index_spec["url"]
+        sha1 = asset_index_spec["sha1"]
+        fpath = self.launcher.get_path(Directory.ASSET_INDEXES, "{}.json".format(iid))
+        if fpath.exists() and file_sha1(fpath) == sha1:
             logger.debug("Using cached asset index, hash matches vspec")
             with open(fpath) as fp:
                 return json.load(fp)
@@ -176,6 +206,14 @@ class Version:
         except requests.ConnectionError:
             die("Failed to retrieve asset index.")
 
+    def get_raw_asset_index_nodl(self, id_):
+        fpath = self.launcher.get_path(Directory.ASSET_INDEXES, "{}.json".format(id_))
+        if fpath.exists():
+            with open(fpath) as fp:
+                return json.load(fp)
+        else:
+            die("Asset index specified in 'assets' not available.")
+
     def get_libraries(self, java_info):
         if java_info is not None:
             key = java_info.get("java.home", None)
@@ -203,14 +241,14 @@ class Version:
         dlspec = self.vspec.downloads.get("client", None)
         if dlspec is None:
             logger.debug("jarfile dlspec not availble, skipping hash check.")
-            if not os.path.exists(self.jarfile):
+            if not self.jarfile.exists():
                 die("jarfile does not exist and can not be downloaded.")
             return
 
         logger.debug("Checking jarfile.")
         if (
             force
-            or not os.path.exists(self.jarfile)
+            or not self.jarfile.exists()
             # The fabric-installer places an empty jarfile here, due to some
             # quirk of an old (git blame 2 years) version of the vanilla launcher.
             # https://github.com/FabricMC/fabric-installer/blob/master/src/main/java/net/fabricmc/installer/client/ClientInstaller.java#L49
@@ -218,9 +256,7 @@ class Version:
             or (verify_hashes and file_sha1(self.jarfile) != dlspec["sha1"])
         ):
             logger.info(
-                "Jar file ({}) will be downloaded with libraries.".format(
-                    self.version_name
-                )
+                "Jar file ({}) will be downloaded with libraries.".format(self.jarname)
             )
             return dlspec["url"], dlspec.get("size", None)
 
@@ -231,14 +267,15 @@ class Version:
         for library in self.get_libraries(java_info):
             if not library.available:
                 continue
-            basedir = get_filepath("libraries")
+            basedir = self.launcher.get_path(Directory.LIBRARIES)
             abspath = library.get_abspath(basedir)
-            ok = os.path.isfile(abspath) and os.path.getsize(abspath) > 0
+            ok = abspath.is_file() and os.path.getsize(abspath) > 0
             if verify_hashes and library.sha1 is not None:
                 ok = ok and file_sha1(abspath) == library.sha1
             if not ok and not library.url:
                 logger.error(
-                    f"Library {library.libname} is missing or corrupt and has no download url."
+                    f"Library {library.filename} is missing or corrupt "
+                    "and has no download url."
                 )
                 continue
             if force or not ok:
@@ -251,28 +288,32 @@ class Version:
             logger.info("Downloading {} libraries.".format(len(q)))
         if not q.download():
             logger.error(
-                "Some libraries failed to download. If they are part of a non-vanilla profile, the original installer may need to be used."
+                "Some libraries failed to download. If they are part of a non-vanilla "
+                "profile, the original installer may need to be used."
             )
 
-    def _populate_virtual_assets(self, where):
-        for name, obj in self.raw_asset_index["objects"].items():
+    def _populate_virtual_assets(self, asset_index, where):
+        for name, obj in asset_index["objects"].items():
             sha = obj["hash"]
-            objpath = get_filepath("assets", "objects", sha[0:2], sha)
-            path = os.path.join(where, *name.split("/"))
+            objpath = self.launcher.get_path(Directory.ASSET_OBJECTS, sha[0:2], sha)
+            path = where / PurePath(*name.split("/"))
             # Maybe check file hash first? Would that be faster?
-            os.makedirs(os.path.dirname(path), exist_ok=True)
-            shutil.copy(get_filepath(objpath), os.path.join(where, path))
+            path.parent.mkdir(parents=True, exist_ok=True)
+            shutil.copy(objpath, path)
 
     def get_virtual_asset_path(self):
-        return get_filepath("assets", "virtual", self.vspec.assetIndex["id"])
+        return self.launcher.get_path(
+            Directory.ASSET_VIRTUAL, self.vspec.assetIndex["id"]
+        )
 
     def prepare_assets_launch(self, gamedir):
-        is_map_resources = self.raw_asset_index.get("map_to_resources", False)
+        launch_asset_index = self.get_raw_asset_index_nodl(self.vspec.assets)
+        is_map_resources = launch_asset_index.get("map_to_resources", False)
         if is_map_resources:
             logger.info("Mapping resources")
-            where = os.path.join(gamedir, "resources")
+            where = gamedir / "resources"
             logger.debug("Resources path: {}".format(where))
-            self._populate_virtual_assets(where)
+            self._populate_virtual_assets(launch_asset_index, where)
 
     def download_assets(self, verify_hashes=False, force=False):
         """Downloads missing assets."""
@@ -285,11 +326,11 @@ class Version:
 
         is_virtual = self.raw_asset_index.get("virtual", False)
 
-        fileset = set(recur_files(get_filepath("assets")))
+        fileset = set(recur_files(self.assets_root))
         q = DownloadQueue()
-        objpath = get_filepath("assets", "objects")
+        objpath = self.launcher.get_path(Directory.ASSET_OBJECTS)
         for sha in hashes:
-            abspath = os.path.join(objpath, sha[0:2], sha)
+            abspath = objpath / sha[0:2] / sha
             ok = abspath in fileset  # file exists
             if verify_hashes:
                 ok = ok and file_sha1(abspath) == sha
@@ -302,19 +343,20 @@ class Version:
         if len(q) > 0:
             logger.info("Downloading {} assets.".format(len(q)))
         if not q.download():
-            logger.warn("Some assets failed to download.")
+            logger.warning("Some assets failed to download.")
 
         if is_virtual:
             logger.info("Copying virtual assets")
             where = self.get_virtual_asset_path()
             logger.debug("Virtual asset path: {}".format(where))
-            self._populate_virtual_assets(where)
+            self._populate_virtual_assets(self.raw_asset_index, where)
 
     def prepare(self, java_info=None, verify_hashes=False):
         if not java_info:
-            java_info = get_java_info(Env.gconf.get("java.path"))
+            java_info = get_java_info(self.launcher.global_config.get("java.path"))
         self.download_libraries(java_info, verify_hashes)
-        self.download_assets(verify_hashes)
+        if hasattr(self, "raw_asset_index"):
+            self.download_assets(verify_hashes)
 
     def prepare_launch(self, gamedir, java_info, verify_hahes=False):
         self.prepare(java_info, verify_hahes)
@@ -322,9 +364,11 @@ class Version:
 
 
 class VersionManager:
-    MANIFEST_URL = "https://launchermeta.mojang.com/mc/game/version_manifest.json"
+    MANIFEST_URL = "https://launchermeta.mojang.com/mc/game/version_manifest_v2.json"
 
-    def __init__(self):
+    def __init__(self, launcher):
+        self.launcher = launcher
+        self.versions_root = launcher.get_path(Directory.VERSIONS)
         self.manifest = self.get_manifest()
 
     def resolve_version_name(self, v):
@@ -338,34 +382,32 @@ class VersionManager:
         return v
 
     def get_manifest(self):
-        manifest_filepath = get_filepath("versions", "manifest.json")
+        manifest_filepath = self.launcher.get_path(Directory.VERSIONS, "manifest.json")
         try:
             m = requests.get(self.MANIFEST_URL).json()
             with open(manifest_filepath, "w") as mfile:
                 json.dump(m, mfile, indent=4, sort_keys=True)
             return m
         except requests.ConnectionError:
-            logger.warn(
+            logger.warning(
                 "Failed to retrieve version_manifest. "
                 "Check your internet connection."
             )
             try:
                 with open(manifest_filepath) as mfile:
-                    logger.warn("Using cached version_manifest.")
+                    logger.warning("Using cached version_manifest.")
                     return json.load(mfile)
             except FileNotFoundError:
-                logger.warn("Cached version manifest not available.")
+                logger.warning("Cached version manifest not available.")
                 raise RuntimeError("Failed to retrieve version manifest.")
 
     def version_list(self, vtype=VersionType.RELEASE, local=False):
         r = [v["id"] for v in self.manifest["versions"] if vtype.match(v["type"])]
         if local:
-            import os
-
             r += sorted(
-                "{} [local]".format(name)
-                for name in os.listdir(get_filepath("versions"))
-                if os.path.isdir(get_filepath("versions", name))
+                "{} [local]".format(path.name)
+                for path in self.versions_root.iterdir()
+                if not path.name.startswith(".") and path.is_dir()
             )
         return r
 
@@ -376,4 +418,4 @@ class VersionManager:
             if ver["id"] == name:
                 version_manifest = ver
                 break
-        return Version(name, self, version_manifest)
+        return Version(name, self.launcher, version_manifest)
